@@ -11,7 +11,7 @@ from copy import copy
 
 from jupyter_core.application import JupyterApp, base_flags, base_aliases
 
-from traitlets import Bool, Unicode
+from traitlets import Bool, Instance, Unicode
 
 from .commands import (
     install_extension, uninstall_extension, list_extensions,
@@ -19,6 +19,7 @@ from .commands import (
     link_package, unlink_package, build, get_app_version, HERE,
     update_extension,
 )
+from .coreconfig import CoreConfig
 from .debuglog import DebugLogFileMixin
 
 
@@ -53,8 +54,12 @@ uninstall_flags['all'] = (
 aliases = dict(base_aliases)
 aliases['app-dir'] = 'BaseExtensionApp.app_dir'
 aliases['dev-build'] = 'BaseExtensionApp.dev_build'
+aliases['minimize'] = 'BaseExtensionApp.minimize'
 aliases['debug-log-path'] = 'DebugLogFileMixin.debug_log_path'
 
+install_aliases = copy(aliases)
+install_aliases['pin-version-as'] = 'InstallLabExtensionApp.pin'
+
 VERSION = get_app_version()
 
 
@@ -63,14 +68,20 @@ class BaseExtensionApp(JupyterApp, DebugLogFileMixin):
     flags = flags
     aliases = aliases
 
+    # Not configurable!
+    core_config = Instance(CoreConfig, allow_none=True)
+
     app_dir = Unicode('', config=True,
         help="The app directory to target")
 
     should_build = Bool(True, config=True,
         help="Whether to build the app after the action")
 
-    dev_build = Bool(True, config=True,
-        help="Whether to build in dev mode (defaults to dev mode)")
+    dev_build = Bool(None, allow_none=True, config=True,
+        help="Whether to build in dev mode. Defaults to True (dev mode) if there are any locally linked extensions, else defaults to False (prod mode).")
+
+    minimize = Bool(True, config=True,
+        help="Whether to use a minifier during the Webpack build (defaults to True). Only affects production builds.")
 
     should_clean = Bool(False, config=True,
         help="Whether temporary files should be cleaned up after building jupyterlab")
@@ -81,9 +92,17 @@ class BaseExtensionApp(JupyterApp, DebugLogFileMixin):
         with self.debug_logging():
             ans = self.run_task()
             if ans and self.should_build:
-                command = 'build:prod' if not self.dev_build else 'build'
+                parts = ['build']
+                parts.append('none' if self.dev_build is None else
+                             'dev' if self.dev_build else
+                             'prod')
+                if self.minimize:
+                    parts.append('minimize')
+                command = ':'.join(parts)
+
                 build(app_dir=self.app_dir, clean_staging=self.should_clean,
-                      logger=self.log, command=command)
+                      logger=self.log, command=command,
+                      core_config=self.core_config)
 
     def run_task(self):
         pass
@@ -94,13 +113,38 @@ class BaseExtensionApp(JupyterApp, DebugLogFileMixin):
 
 
 class InstallLabExtensionApp(BaseExtensionApp):
-    description = "Install labextension(s)"
+    description = """Install labextension(s)
+    
+     Usage
+    
+        jupyter labextension install [--pin-version-as <alias,...>] <package...>
+    
+    This installs JupyterLab extensions similar to yarn add or npm install.
+    
+    Pass a list of comma seperate names to the --pin-version-as flag
+    to use as alises for the packages providers. This is useful to
+    install multiple versions of the same extension.
+    These can be uninstalled with the alias you provided
+    to the flag, similar to the "alias" feature of yarn add.
+    """
+    aliases = install_aliases
+
+    pin = Unicode('', config=True,
+        help="Pin this version with a certain alias")
 
     def run_task(self):
+        pinned_versions = self.pin.split(',')
         self.extra_args = self.extra_args or [os.getcwd()]
         return any([
-            install_extension(arg, self.app_dir, logger=self.log)
-            for arg in self.extra_args
+            install_extension(
+                arg,
+                self.app_dir,
+                logger=self.log,
+                core_config=self.core_config,
+                # Pass in pinned alias if we have it
+                pin=pinned_versions[i] if i < len(pinned_versions) else None
+            )
+            for i, arg in enumerate(self.extra_args)
         ])
 
 
@@ -116,9 +160,13 @@ class UpdateLabExtensionApp(BaseExtensionApp):
             self.log.warn('Specify an extension to update, or use --all to update all extensions')
             return False
         if self.all:
-            return update_extension(all_=True, app_dir=self.app_dir, logger=self.log)
+            return update_extension(
+                all_=True, app_dir=self.app_dir, logger=self.log,
+                core_config=self.core_config)
         return any([
-            update_extension(name=arg, app_dir=self.app_dir, logger=self.log)
+            update_extension(
+                name=arg, app_dir=self.app_dir, logger=self.log,
+                core_config=self.core_config)
             for arg in self.extra_args
         ])
 
@@ -137,7 +185,9 @@ class LinkLabExtensionApp(BaseExtensionApp):
     def run_task(self):
         self.extra_args = self.extra_args or [os.getcwd()]
         return any([
-            link_package(arg, self.app_dir, logger=self.log)
+            link_package(
+                arg, self.app_dir, logger=self.log,
+                core_config=self.core_config)
             for arg in self.extra_args
         ])
 
@@ -148,7 +198,9 @@ class UnlinkLabExtensionApp(BaseExtensionApp):
     def run_task(self):
         self.extra_args = self.extra_args or [os.getcwd()]
         return any([
-            unlink_package(arg, self.app_dir, logger=self.log)
+            unlink_package(
+                arg, self.app_dir, logger=self.log,
+                core_config=self.core_config)
             for arg in self.extra_args
         ])
 
@@ -163,7 +215,9 @@ class UninstallLabExtensionApp(BaseExtensionApp):
     def run_task(self):
         self.extra_args = self.extra_args or [os.getcwd()]
         return any([
-            uninstall_extension(arg, all_=self.all, app_dir=self.app_dir, logger=self.log)
+            uninstall_extension(
+                arg, all_=self.all, app_dir=self.app_dir, logger=self.log,
+                core_config=self.core_config)
             for arg in self.extra_args
         ])
 
@@ -172,14 +226,16 @@ class ListLabExtensionsApp(BaseExtensionApp):
     description = "List the installed labextensions"
 
     def run_task(self):
-        list_extensions(self.app_dir, logger=self.log)
+        list_extensions(
+            self.app_dir, logger=self.log, core_config=self.core_config)
 
 
 class EnableLabExtensionsApp(BaseExtensionApp):
     description = "Enable labextension(s) by name"
 
     def run_task(self):
-        [enable_extension(arg, self.app_dir, logger=self.log)
+        [enable_extension(
+            arg, self.app_dir, logger=self.log, core_config=self.core_config)
          for arg in self.extra_args]
 
 
@@ -187,7 +243,8 @@ class DisableLabExtensionsApp(BaseExtensionApp):
     description = "Disable labextension(s) by name"
 
     def run_task(self):
-        [disable_extension(arg, self.app_dir, logger=self.log)
+        [disable_extension(
+            arg, self.app_dir, logger=self.log, core_config=self.core_config)
          for arg in self.extra_args]
 
 
@@ -203,7 +260,8 @@ class CheckLabExtensionsApp(BaseExtensionApp):
             check_extension(
                 arg, self.app_dir,
                 self.should_check_installed_only,
-                logger=self.log)
+                logger=self.log,
+                core_config=self.core_config)
             for arg in self.extra_args)
         if not all_enabled:
             self.exit(1)
