@@ -5,20 +5,20 @@
 # Distributed under the terms of the Modified BSD License.
 import glob
 import json
+import logging
 import os
 import shutil
 import sys
+import subprocess
+import shutil
+import pathlib
+import platform
 from os.path import join as pjoin
+from tempfile import TemporaryDirectory
 from unittest import TestCase
+from unittest.mock import patch
+
 import pytest
-
-try:
-    from unittest.mock import patch
-except ImportError:
-    from mock import patch  # py2
-
-from ipython_genutils import py3compat
-from ipython_genutils.tempdir import TemporaryDirectory
 from notebook.notebookapp import NotebookApp
 from jupyter_core import paths
 
@@ -28,9 +28,9 @@ from jupyterlab.commands import (
     install_extension, uninstall_extension, list_extensions,
     build, link_package, unlink_package, build_check,
     disable_extension, enable_extension, get_app_info,
-    check_extension, _test_overlap, _get_core_data,
-    update_extension,
+    check_extension, _test_overlap, update_extension
 )
+from jupyterlab.coreconfig import CoreConfig, _get_default_core_data
 
 here = os.path.dirname(os.path.abspath(__file__))
 
@@ -56,7 +56,7 @@ class TestExtension(TestCase):
     def tempdir(self):
         td = TemporaryDirectory()
         self.tempdirs.append(td)
-        return py3compat.cast_unicode(td.name)
+        return td.name
 
     def setUp(self):
         # Any TemporaryDirectory objects appended to this list will be cleaned
@@ -125,12 +125,15 @@ class TestExtension(TestCase):
         self.assertEqual(paths.ENV_CONFIG_PATH, [self.config_dir])
         self.assertEqual(paths.ENV_JUPYTER_PATH, [self.data_dir])
         self.assertEqual(
-            commands.get_app_dir(),
+            os.path.realpath(commands.get_app_dir()),
             os.path.realpath(pjoin(self.data_dir, 'lab'))
         )
 
         self.app_dir = commands.get_app_dir()
 
+        # Set pinned extension names
+        self.pinned_packages = ['jupyterlab-test-extension@1.0', 'jupyterlab-test-extension@2.0']
+
     def test_install_extension(self):
         assert install_extension(self.mock_extension) is True
         path = pjoin(self.app_dir, 'extensions', '*.tgz')
@@ -201,6 +204,19 @@ class TestExtension(TestCase):
         assert name not in extensions
         assert not check_extension(name)
 
+    def test_uninstall_all_extensions(self):
+        install_extension(self.mock_extension)
+        install_extension(self.mock_mimeextension)
+        ext_name = self.pkg_names['extension']
+        mime_ext_name = self.pkg_names['mimeextension']
+        assert check_extension(ext_name) is True
+        assert check_extension(mime_ext_name) is True
+        assert uninstall_extension(all_=True) is True
+        extensions = get_app_info(self.app_dir)['extensions']
+        assert ext_name not in extensions
+        assert mime_ext_name not in extensions
+
+
     def test_uninstall_core_extension(self):
         assert uninstall_extension('@jupyterlab/console-extension') is True
         app_dir = self.app_dir
@@ -219,6 +235,58 @@ class TestExtension(TestCase):
         assert '@jupyterlab/console-extension' in extensions
         assert check_extension('@jupyterlab/console-extension')
 
+    def test_install_and_uninstall_pinned(self):
+        """
+        You should be able to install different versions of the same extension with different
+        pinned names and uninstall them with those names.
+        """
+        NAMES = ['test-1', 'test-2']
+        assert install_extension(self.pinned_packages[0], pin=NAMES[0])
+        assert install_extension(self.pinned_packages[1], pin=NAMES[1])
+
+        extensions = get_app_info(self.app_dir)['extensions']
+        assert NAMES[0] in extensions
+        assert NAMES[1] in extensions
+        assert check_extension(NAMES[0])
+        assert check_extension(NAMES[1])
+
+        # Uninstall
+        assert uninstall_extension(NAMES[0])
+        assert uninstall_extension(NAMES[1])
+
+        extensions = get_app_info(self.app_dir)['extensions']
+        assert NAMES[0] not in extensions
+        assert NAMES[1] not in extensions
+        assert not check_extension(NAMES[0])
+        assert not check_extension(NAMES[1])
+
+    @pytest.mark.skipif(platform.system() == 'Windows', reason='running npm pack fails on windows CI')
+    def test_install_and_uninstall_pinned_folder(self):
+        """
+        Same as above test, but installs from a local folder instead of from npm.
+        """
+        # Download each version of the package from NPM:
+        base_dir = pathlib.Path(self.tempdir())
+
+        # The archive file names are printed to stdout when run `npm pack`
+        packages = [
+            subprocess.run(
+                ['npm', 'pack', name],
+                stdout=subprocess.PIPE,
+                universal_newlines=True,
+                check=True,
+                cwd=base_dir
+            ).stdout.strip()
+            for name in self.pinned_packages
+        ]
+
+        shutil.unpack_archive(str(base_dir / packages[0]), str(base_dir / '1'))
+        shutil.unpack_archive(str(base_dir / packages[1]), str(base_dir / '2'))
+        # Change pinned packages to be these directories now, so we install from these folders
+        self.pinned_packages = [str(base_dir / '1' / 'package'), str(base_dir / '2' / 'package')]
+        self.test_install_and_uninstall_pinned()
+
+
     def test_link_extension(self):
         path = self.mock_extension
         name = self.pkg_names['extension']
@@ -356,7 +424,7 @@ class TestExtension(TestCase):
 
     def test_build_custom(self):
         assert install_extension(self.mock_extension) is True
-        build(name='foo', version='1.0', public_url='bar')
+        build(name='foo', version='1.0', static_url='bar')
 
         # check static directory.
         entry = pjoin(self.app_dir, 'static', 'index.out.js')
@@ -369,7 +437,52 @@ class TestExtension(TestCase):
             data = json.load(fid)
         assert data['jupyterlab']['name'] == 'foo'
         assert data['jupyterlab']['version'] == '1.0'
-        assert data['jupyterlab']['publicUrl'] == 'bar'
+        assert data['jupyterlab']['staticUrl'] == 'bar'
+
+    def test_build_custom_minimal_core_config(self):
+        default_config = CoreConfig()
+        core_config = CoreConfig()
+        core_config.clear_packages()
+        logger = logging.getLogger('jupyterlab_test_logger')
+        logger.setLevel('DEBUG')
+        extensions = (
+            '@jupyterlab/application-extension',
+            '@jupyterlab/apputils-extension',
+        )
+        singletons = (
+            "@jupyterlab/application",
+            "@jupyterlab/apputils",
+            "@jupyterlab/coreutils",
+            "@jupyterlab/services",
+        )
+        for name in extensions:
+            semver = default_config.extensions[name]
+            core_config.add(name, semver, extension=True)
+        for name in singletons:
+            semver = default_config.singletons[name]
+            core_config.add(name, semver)
+
+        assert install_extension(self.mock_extension) is True
+        build(core_config=core_config, logger=logger)
+
+        # check static directory.
+        entry = pjoin(self.app_dir, 'static', 'index.out.js')
+        with open(entry) as fid:
+            data = fid.read()
+        assert self.pkg_names['extension'] in data
+
+        pkg = pjoin(self.app_dir, 'static', 'package.json')
+        with open(pkg) as fid:
+            data = json.load(fid)
+        assert list(data['jupyterlab']['extensions'].keys()) == [
+            '@jupyterlab/application-extension',
+            '@jupyterlab/apputils-extension',
+            self.pkg_names['extension'],
+        ]
+        assert data['jupyterlab']['mimeExtensions'] == {}
+        for pkg in data['jupyterlab']['singletonPackages']:
+            if pkg.startswith('@jupyterlab/'):
+                assert pkg in singletons
 
     def test_load_extension(self):
         app = NotebookApp()
@@ -469,7 +582,7 @@ class TestExtension(TestCase):
         assert _test_overlap('<0.6', '0.1') is None
 
     def test_install_compatible(self):
-        core_data = _get_core_data()
+        core_data = _get_default_core_data()
         current_app_dep = core_data['dependencies']['@jupyterlab/application']
         def _gen_dep(ver):
             return { "dependencies": {
